unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, ComCtrls,
  StdCtrls, Buttons, ColorBox, EditBtn;

type

  { TTeroEditor }

  TItem = record
    X, Y, W : Integer;
    iType : Integer;
    ptr : TGraphicControl;
  end;


  TTeroEditor = class(TForm)
    BitBtnLoad: TBitBtn;
    BitBtnAsm: TBitBtn;
    BitBtnSave: TBitBtn;
    BitBtnNew: TBitBtn;
    BitBtnShow: TBitBtn;
    cbAutoReturn: TCheckBox;
    ColorBox1: TColorBox;
    ColorBox2: TColorBox;
    ColorBox3: TColorBox;
    cbNeigbourUp: TComboBox;
    cbNeigbourDown: TComboBox;
    cbNeigbourLeft: TComboBox;
    cbNeigbourRight: TComboBox;
    EdCurr: TEdit;
    FileNameEdit1: TFileNameEdit;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    ImageBarrierBlink: TImage;
    ImageGranade: TImage;
    ImageWaterBar: TImage;
    ImageWater: TImage;
    ImageRaft: TImage;
    ImageScorpion: TImage;
    ImageDoorM: TImage;
    ImageDoorG: TImage;
    ImageDoorR: TImage;
    ImageKeyR: TImage;
    ImageKeyM: TImage;
    ImageDyn: TImage;
    ImageKeyG: TImage;
    ImageMediKit: TImage;
    ImageMoth: TImage;
    ImageVictim: TImage;
    ImageSnake: TImage;
    ImageSpider: TImage;
    ImageBat: TImage;
    ImagePlayer: TImage;
    ImageLight: TImage;
    ImageWall: TImage;
    ImageBarrier: TImage;
    StaticText2: TStaticText;
    StaticText3: TStaticText;
    StaticText4: TStaticText;
    StaticText5: TStaticText;
    StaticText6: TStaticText;
    StaticText7: TStaticText;
    StaticText9: TStaticText;
    StatusBar1: TStatusBar;
    TeroImg: TImageList;
    Panel1: TPanel;
    ShapeWall: TShape;
    StaticText1: TStaticText;
    TabScreen: TTabControl;
    procedure BitBtnAsmClick(Sender: TObject);
    procedure BitBtnNewClick(Sender: TObject);
    procedure BitBtnShowClick(Sender: TObject);
    procedure BitBtnLoadClick(Sender: TObject);
    procedure BitBtnSaveClick(Sender: TObject);
    procedure cbNeigbourChange(Sender: TObject);
    procedure cbNeigbourMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ColorBoxChange(Sender: TObject);
    procedure ColorBox1GetColors(Sender: TCustomColorBox; Items: TStrings);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ImageDynEndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure ImageDynMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageDynMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ImageDynMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageItemClick(Sender: TObject);
    procedure ShapeWallMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ShapeWallMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer
      );
    procedure ShapeWallMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TabScreenChange(Sender: TObject);
  private
    MouseOriginatePosition : TPoint;
    PressedMouseButton : TMouseButton;
    nb : array[1..32] of TShape;

    procedure BackLink(i, a, p: Integer);
    procedure DrawNeighbours(above, below: Integer);
    procedure GreyBox(y, x, w: Integer);
    procedure stat(s:String);
    function CreateItem(t,x,y,w:Integer):integer;
    Procedure DeleteItem(idx:Integer);
    procedure LoadImage(im:Timage;h,w:integer;fn:string);
    function ScreenToString:String;
    procedure StringToScreen(s:string);

  public
    item : Array [1..100] of TItem;
    scrn : Array [1..20] of String;
    CurrScrn :  Integer;
  end;

var
  TeroEditor: TTeroEditor;

implementation

{$R *.lfm}

{ TTeroEditor }

uses Math, Clipbrd;

function toxhex(i:integer):char;
begin
  if i>9 then toxhex:=chr(i+55)
         else toxhex:=chr(i+48);
end;

function xhex(s: string;p:integer):integer;
var v : Integer;
begin
  if (p<1) or (p>length(s)) then xhex:=1 else begin
    v := byte(s[p]) - 48;
    if v>10 then dec(v,7);
    xhex := v;
  end;
end;

Procedure ClearItems;
var i : Integer;
begin
  for i:=1 to 100 do TeroEditor.DeleteItem(i);
end;

Procedure TTeroEditor.DeleteItem(idx:Integer);
begin
  item[idx].X:=0;
  item[idx].Y:=0;
  if item[idx].iType=12
    then
      TShape(item[idx].ptr).Free
    else
      TImage(item[idx].ptr).Free;
  item[idx].ptr   := NIL;
  item[idx].iType := 0;
end;

Function  TTeroEditor.CreateItem(t,x,y,w:Integer):integer;
var idx : Integer;
    abrt : boolean;
begin
   // elements can only be present once: Moth #7, Victim #2, Raft #8
   if t in [2,7,8] then begin
     abrt := false;
     for idx:=1 to 100 do if item[idx].iType = t then abrt:=true;
     if abrt then begin
       ShowMessage('Item already on screen.');
       exit;
     end;
   end;

   idx:=1; // find empty slot
   while item[idx].iType<>0 do inc(idx);
   CreateItem := idx;
   item[idx].w := w;
   item[idx].x := x;
   item[idx].y := y;
   item[idx].iType:=t;

   if item[idx].iType=12 then begin // Wall = TShape
      item[idx].ptr := TShape.Create(Panel1);
      with item[idx].ptr as TShape do begin
         Shape := stRectangle;
         DragMode:= dmAutomatic;
         brush.Color:=clWhite;
         height := 200;
         Width  := item[idx].w * 40;
         left   := item[idx].x * 40 - 40;
         Top    := item[idx].y * 40 - 40;
         Visible:= true;
         Parent := Panel1;
         tag    := idx;
         OnMouseDown:=@ShapeWallMouseDown;
         OnMouseMove:=@ShapeWallMouseMove;
         OnMouseUp  :=@ShapeWallMouseUp;
      end;
   end else begin // Enemy = TImage
      item[idx].ptr := TImage.Create(Panel1);
      with item[idx].ptr as TImage do begin
        left   := item[idx].x * 40 - 40;
        Top    := item[idx].y * 40 - 40;
        Visible:= true;
        Parent := Panel1;
        tag    := idx;
        DragMode   := dmAutomatic;
        OnMouseDown:=@ImageDynMouseDown;
        OnMouseMove:=@ImageDynMouseMove;
        OnMouseUp  :=@ImageDynMouseUp;
        OnEndDrag  :=@ImageDynEndDrag;
        case item[idx].iType of
          1 : LoadImage(TImage(item[idx].ptr),139,80,'Pic\Player.png');
          2 : LoadImage(TImage(item[idx].ptr),80,80, 'Pic\victim.png');
          3 : LoadImage(TImage(item[idx].ptr),80,40, 'Pic\Lamp.png');
          4 : LoadImage(TImage(item[idx].ptr),120,80,'Pic\Bat.png');
          5 : LoadImage(TImage(item[idx].ptr),80,80, 'Pic\Spider.png');
          6 : LoadImage(TImage(item[idx].ptr),40,80, 'Pic\Snake.png');
          7 : LoadImage(TImage(item[idx].ptr),200,200,'Pic\Moth.png');
          8 : LoadImage(TImage(item[idx].ptr),40,80, 'Pic\Raft.png');
          9 : LoadImage(TImage(item[idx].ptr),200,80,'Pic\Barrier.png');
         10 : LoadImage(TImage(item[idx].ptr),200,40,'Pic\door-magenta.png');
         11 : LoadImage(TImage(item[idx].ptr),40,40, 'Pic\Key-magenta.png');
         13 : LoadImage(TImage(item[idx].ptr),40,40, 'Pic\medikit.png');
         14 : LoadImage(TImage(item[idx].ptr),200,40,'Pic\door-green.png');
         15 : LoadImage(TImage(item[idx].ptr),40,40, 'Pic\Key-green.png');
         16 : LoadImage(TImage(item[idx].ptr),200,40,'Pic\door-red.png');
         17 : LoadImage(TImage(item[idx].ptr),40,40, 'Pic\Key-red.png');
         18 : LoadImage(TImage(item[idx].ptr),40,40, 'Pic\Scorpion.png');
         20 : LoadImage(TImage(item[idx].ptr),200,80,'Pic\Barrier-red.png');
         21 : LoadImage(TImage(item[idx].ptr),80,40, 'Pic\Granade.png');
        end;
        if item[idx].iType in [2,7,9,10,14,16,20] then Stretch:= true;
      end;

   end;
end;

procedure TTeroEditor.LoadImage(im:Timage;h,w:integer;fn:string);
begin
  im.Height:=h;
  im.Width :=w;
  im.Picture.LoadFromFile(fn);
end;

procedure TTeroEditor.stat(s:String);
begin
  StatusBar1.Panels.Items[1].Text:=s;
end;

function TTeroEditor.ScreenToString:String;
var s:string;
    i,j,c:integer;
    wl : Array[0..2,1..32] of boolean;
    it,wall,wlx : String;
    wc : Array[0..2] of integer;
    bl : Boolean;

begin
  s:='';
  s:=s+ToXHex(StrToInt(cbNeigbourUp.text));
  s:=s+ToXHex(StrToInt(cbNeigbourRight.text));
  s:=s+ToXHex(StrToInt(cbNeigbourDown.text));
  s:=s+ToXHex(StrToInt(cbNeigbourLeft.text));
  for i:=0 to 2 do
    for j:=1 to 32 do wl[i,j] := false;
  it:='';
  for i:=1 to 100 do with item[i] do if iType>0 then begin
    if iType=12 then begin // wall
      for j:=1 to W do wl[y DIV 5,min(x+j-1,32)]:=true;
    end else begin
      it:=it+chr(iType+96)+toxhex(y)+toxhex(x);
    end;
  end;

  wc[0] := ColorBox1.ItemIndex;
  wc[1] := ColorBox2.ItemIndex;
  wc[2] := ColorBox3.ItemIndex;
  wall  := '';

  for i:=0 to 2 do begin
    wall := wall + chr(wc[i]+97);
    j:=2; c:=1;
    wlx := '';
    bl := wl[i,1];
    repeat
      while (j<=32) and (wl[i,j]=bl) do begin
        inc(j);
        inc(c);
      end;
      wlx := wlx + toxhex(c);
      c  := 0;
      bl := not bl;
    until j>=32;
    if wl[i,1] then wall := wall + wlx
               else wall := wall + '0' + wlx;
  end;
  if ImageWaterBar.Visible then it := it + 's00';
  ScreenToString := s+wall+it;
end;

procedure TTeroEditor.ShapeWallMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseOriginatePosition.X := X;
  MouseOriginatePosition.Y := Y;
  PressedMouseButton := Button;
end;

procedure TTeroEditor.ImageDynMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseOriginatePosition.X := X;
  MouseOriginatePosition.Y := Y;
end;

procedure TTeroEditor.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  ClearItems;
  CurrScrn := 1;
  for i:=1 to 20 do scrn[i]:='0000';
end;

procedure TTeroEditor.ColorBox1GetColors(Sender: TCustomColorBox;
  Items: TStrings);
begin
  with Sender do begin
    AddItem('  3 Medium Green',TObject($42c821));
    AddItem('  4 Light Green',TObject($78dc5e));
    AddItem('  5 Blue',TObject($ed5554));
    AddItem('  6 Light Blue',TObject($fc767d));
    AddItem('  7 Dark Red',TObject($4d52d4));
    AddItem('  8 Cyan',TObject($f5eb42));
    AddItem('  9 Red',TObject($5455fc));
    AddItem('10 Light Red',TObject($7879ff));
    AddItem('11 Dark Yellow',TObject($54c1d4));
    AddItem('12 Light Yellow',TObject($80cee6));
    AddItem('13 Dark Green',TObject($3bb021));
    AddItem('14 Magenta',TObject($ba5bc9));
    AddItem('15 Gray',TObject($cccccc));
    AddItem('16 White',TObject($ffffff));
    AddItem('17 Blinking',TObject($ca6485));
  end;
end;

procedure TTeroEditor.BitBtnShowClick(Sender: TObject);
var s : string;
begin
  s := ScreenToString;
  clipboard.AsText:=s;
  stat('Screen definition string copied to Clipboard, length '+inttostr(length(s))+'.');
  ShowMessage(s + ' copied to Clipboard');
end;

procedure TTeroEditor.BitBtnNewClick(Sender: TObject);
begin
    if MessageDlg( 'Start new','Are you sure to drop current Level and start fresh?',mtConfirmation, [mbYes, mbNo],0)<>mrYes then exit;
    FormCreate(NIL);
    StringToScreen(scrn[1]);
    TabScreen.TabIndex:=0;
    CurrScrn:=1;
    EdCurr.Text:=IntToStr(1);
end;

procedure TTeroEditor.BitBtnAsmClick(Sender: TObject);
var t : textfile;
    fn : String;
    i, last : integer;
begin
  fn := FileNameEdit1.Text+'.asm';
  scrn[CurrScrn] := ScreenToString;
  for i:=1 to 20 do if scrn[i]<>'0000' then last := i;
  AssignFile(t,fn);
  rewrite(t);
  writeln(t,'*      ROOM '+ExtractFileName(FileNameEdit1.Text));
  writeln(t,'*');
  for i:=1 to last do
    if length(scrn[i])<40
      then writeln(t,'       TEXT '''+scrn[i]+',''')
      else begin
        writeln(t,'       TEXT '''+copy(scrn[i],1,40)+'''');
        writeln(t,'       TEXT '''+copy(scrn[i],41,40)+',''')
      end;
  writeln(t,'       BYTE 44');  // last screen
  writeln(t,'');
  CloseFile(t);
  stat('Assembler File '+fn+' exported.');

end;

procedure TTeroEditor.BitBtnLoadClick(Sender: TObject);
var t : textfile;
    fn : String;
    i : integer;
begin
  fn := FileNameEdit1.Text;
  if MessageDlg( 'Load file','Are you sure to drop current Level and load from file?',mtConfirmation, [mbYes, mbNo],0)<>mrYes then exit;
  if FileExists(fn) then begin
    FormCreate(NIL);
    StringToScreen(scrn[1]);
    TabScreen.TabIndex:=0;
    CurrScrn:=1;
    EdCurr.Text:=IntToStr(1);
    for i:=1 to 20 do scrn[i]:='0000';
    i:=1;
    AssignFile(t,fn);
    reset(t);
    while not eof(t) do begin
      readln(t,scrn[i]);
      inc(i);
    end;
    CloseFile(t);
    stat('File '+fn+' read.');
    StringToScreen(scrn[1]);
    TabScreen.TabIndex:=0;
    CurrScrn:=1;
    EdCurr.Text:=IntToStr(CurrScrn);
  end else stat('File not found: '+fn);
end;

procedure TTeroEditor.BitBtnSaveClick(Sender: TObject);
var t : textfile;
    fn : String;
    i, last : integer;
begin
  fn := FileNameEdit1.Text;
  scrn[CurrScrn] := ScreenToString;
  for i:=1 to 20 do if scrn[i]<>'0000' then last := i;
  AssignFile(t,fn);
  rewrite(t);
  for i:=1 to last do
      writeln(t,scrn[i]);
  CloseFile(t);
  stat('File '+fn+' saved.');
end;

procedure TTeroEditor.cbNeigbourChange(Sender: TObject);
begin
    scrn[CurrScrn] := ScreenToString;
    DrawNeighbours(cbNeigbourUp.ItemIndex,cbNeigbourDown.ItemIndex);
end;

procedure TTeroEditor.cbNeigbourMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button=mbRight) and (TComboBox(Sender).ItemIndex<>0) then
     TabScreen.TabIndex:=TComboBox(Sender).ItemIndex-1;
end;

procedure TTeroEditor.ColorBoxChange(Sender: TObject);
var i:integer;
begin
  for i:=1 to 100 do if item[i].iType=12 then begin
    case item[i].ptr.Top DIV 200 of
      0 :  TShape(item[i].ptr).Brush.Color := ColorBox1.Selected;
      1 :  TShape(item[i].ptr).Brush.Color := ColorBox2.Selected;
      else TShape(item[i].ptr).Brush.Color := ColorBox3.Selected;
    end;
     item[i].ptr.Invalidate;
  end;
end;

procedure TTeroEditor.FormActivate(Sender: TObject);
begin
   if ColorBox1.ItemIndex=-1 then ColorBox1.ItemIndex:=2;
   if ColorBox2.ItemIndex=-1 then ColorBox2.ItemIndex:=10;
   if ColorBox3.ItemIndex=-1 then ColorBox3.ItemIndex:=3;
end;

procedure TTeroEditor.ImageDynEndDrag(Sender, Target: TObject; X, Y: Integer);
begin
  if (TImage(Sender).Top>=680) or (TImage(Sender).left>=1280) then begin
    stat('Deleted item type '+intToStr(item[TImage(Sender).Tag].iType));
    item[TImage(Sender).Tag].iType:=0;
  end;
end;

procedure TTeroEditor.ImageDynMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
   if Mouse.IsDragging then
    begin
      TImage(Sender).Left := Max(TImage(Sender).Left + X - MouseOriginatePosition.X,0);
      TImage(Sender).Top  := Max(TImage(Sender).Top  + Y - MouseOriginatePosition.Y,0);
    end;
end;

procedure TTeroEditor.ImageDynMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var t : integer;
begin
  TImage(Sender).Left := ((TImage(Sender).Left + 20) DIV 40) * 40;
  TImage(Sender).Top  := ((TImage(Sender).Top  + 20) DIV 40) * 40;
  t := TImage(Sender).tag;
  item[t].x :=(TImage(Sender).Left DIV 40) + 1;
  item[t].y :=(TImage(Sender).Top  DIV 40) + 1;
  stat('Item '+inttoStr(t)+' new Position: ('+IntToStr(item[t].x)+','+IntToStr(item[t].y)+')');

  if  item[t].y > 17 then begin
    DeleteItem(t);
    stat('Item '+inttoStr(t)+' deleted.')
  end;

end;

procedure TTeroEditor.ImageItemClick(Sender: TObject);
begin
  if TImage(Sender).tag = 19 then ImageWaterBar.visible := not ImageWaterBar.visible
     else CreateItem(TImage(Sender).tag,15,7,2);
end;


procedure TTeroEditor.ShapeWallMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
   if Mouse.IsDragging then
    begin
      TShape(Sender).Left := Max(TShape(Sender).Left + X - MouseOriginatePosition.X,0);
      TShape(Sender).Top  := Max(TShape(Sender).Top  + Y - MouseOriginatePosition.Y,0);
    end;
   if PressedMouseButton=mbRight then
     TShape(Sender).Width:=X+20  ;

end;

procedure TTeroEditor.ShapeWallMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var t : integer;
begin
  // snap to grid
  TShape(Sender).Left := ((TShape(Sender).Left + 20)  DIV 40)  * 40;
  TShape(Sender).Top  := ((TShape(Sender).Top  + 100) DIV 200) * 200 + 80;

  TShape(Sender).Height  := 198;
  case TShape(Sender).Top DIV 200 of
   0 : TShape(Sender).Brush.Color := ColorBox1.Selected;
   1 : TShape(Sender).Brush.Color := ColorBox2.Selected;
   else TShape(Sender).Brush.Color:= ColorBox3.Selected;
  end;

  PressedMouseButton := mbExtra2;
  TShape(Sender).Width:= Max(((TShape(Sender).Width+10) DIV 40) * 40,80);

  t := integer(TImage(Sender).tag);
  item[t].x :=(TShape(Sender).Left  DIV 40) + 1;
  item[t].y :=(TShape(Sender).Top   DIV 40) + 1;
  item[t].w :=(TShape(Sender).Width DIV 40);

  stat('Wall '+inttoStr(t)+' new Position: ('+IntToStr(item[t].x)+','+IntToStr(item[t].y)+'), width '+IntToStr(item[t].w));

  if  item[t].y > 14 then begin
    DeleteItem(t);
    stat('Wall '+inttoStr(t)+' deleted.')
  end;
end;

procedure TTeroEditor.BackLink(i,a,p:Integer);
begin
  if xhex(scrn[a],p)=0 then begin
    scrn[a,p]:=toxhex(i);
    stat('Auto-Return from '+inttostr(a)+' to '+inttostr(i)+' added.');
  end;
end;

procedure TTeroEditor.TabScreenChange(Sender: TObject);
var i,a:integer;
begin
  scrn[CurrScrn] := ScreenToString;
  CurrScrn:=TabScreen.TabIndex+1;
  EdCurr.Text:=IntToStr(CurrScrn);
  if cbAutoReturn.Checked then
    for i:=1 to 20 do if length(scrn[i])>3 then begin
      a := xhex(scrn[i],1); if a<>0 then Backlink(i,a,3);
      a := xhex(scrn[i],2); if a<>0 then Backlink(i,a,4);
      a := xhex(scrn[i],3); if a<>0 then Backlink(i,a,1);
      a := xhex(scrn[i],4); if a<>0 then Backlink(i,a,2);
    end;
  StringToScreen(scrn[CurrScrn]);
  if CurrScrn=1 then begin
     ImagePlayer.Enabled:=true;
     ImagePlayer.Picture.LoadFromFile('Pic\Player-Square.png');
  end else begin
    ImagePlayer.Enabled:=false;
    ImagePlayer.Picture.LoadFromFile('Pic\Player-Square-bw.png');
  end;
end;

procedure TTeroEditor.StringToScreen(s:string);
var i,p,x,y,w : integer;
begin
  // clear current screen
  ClearItems;
  ImageWaterBar.Visible:=false;
  // set neighbors
  if length(s)<4 then s:='0000';
  cbNeigbourUp.ItemIndex   :=xhex(s,1);
  cbNeigbourRight.ItemIndex:=xhex(s,2);
  cbNeigbourDown.ItemIndex :=xhex(s,3);
  cbNeigbourLeft.ItemIndex :=xhex(s,4);

  if length(s)>5 then begin
    // draw walls
    y := 3; p := 5;
    repeat
      x := 1;
      case y DIV 5 of
       0 : ColorBox1.ItemIndex:=byte(s[p])-97;
       1 : ColorBox2.ItemIndex:=byte(s[p])-97;
       2 : ColorBox3.ItemIndex:=byte(s[p])-97;
      end;
      inc(p);
      repeat
      w := xhex(s,p);
      inc(p);
      if w>0 then begin
        i:=CreateItem(12,x,y,w);
        inc(x,w);
        case y DIV 5 of
         0 : TShape(item[i].ptr).Brush.Color := ColorBox1.Selected;
         1 : TShape(item[i].ptr).Brush.Color := ColorBox2.Selected;
         else TShape(item[i].ptr).Brush.Color:= ColorBox3.Selected;
        end;

      end;
      if x<32 then begin
        inc(x,xhex(s,p));
        inc(p)
      end;
      until x>=32;
      y := y + 5;
    until y>17;

    // Beasts and Items
    while p<length(s) do begin
      if s[p]='s'
        then ImageWaterBar.Visible:=true
        else CreateItem(byte(s[p])-96,xhex(s,p+2),xhex(s,p+1),2);
      inc(p,3);
    end;
  end;
  DrawNeighbours(xhex(s,1),xhex(s,3));
end;

procedure TTeroEditor.DrawNeighbours(above,below: Integer);
var i,x,w : Integer;
    s : String;
begin
  for i:=1 to 32 do begin
    nb[i].free;
    nb[i] := NIL;
  end;

  if above<>0 then begin
    s := scrn[above];
    if length(s)>8 then begin
      i:=6;
      while s[i]<'a' do inc(i); // find second color code
      inc(i);
      while s[i]<'a' do inc(i); // find third color code
      x:=1;
      inc(i);
      repeat
        w:=xhex(s,i);
        if w>0 then GreyBox(2,x,w);
        inc(x,w);
        inc(i);
        if x<32 then begin
          w:=xhex(s,i);
          inc(x,w);
          inc(i)
        end;
      until x>=32;
    end;
  end;

  if below<>0 then begin
    s := scrn[below];
    if length(s)>8 then begin
      i:=6;
      x:=1;
      repeat
        w:=xhex(s,i);
        if w>0 then GreyBox(18,x,w);
        inc(x,w);
        inc(i);
        if x<32 then begin
          w:=xhex(s,i);
          inc(x,w);
          inc(i);
        end;
      until x>=32;
    end;
  end;


end;

procedure TTeroEditor.GreyBox(y,x,w:Integer);
var i:integer;
begin
  i:=1;
  while nb[i]<>NIL do inc(i);
  nb[i] := Tshape.create(Panel1);
  with nb[i] do begin
         Shape := stRectangle;
         brush.Color:=clGray;
         height := 38;
         Width  := w * 40;
         left   := x * 40 - 40;
         Top    := y * 40 - 40;
         Visible:= true;
         Parent := Panel1;
  end;
end;

end.

